@@ -14,11 +14,8 @@ from django.core.urlresolvers import reverse_lazy
 from mama_cas.models import ServiceTicket
 from mama_cas.models import ProxyTicket
 from mama_cas.models import ProxyGrantingTicket
-from mama_cas.exceptions import InvalidRequestError
-from mama_cas.exceptions import InvalidTicketError
-from mama_cas.exceptions import InvalidServiceError
-from mama_cas.exceptions import InternalError
-from mama_cas.exceptions import BadPGTError
+from mama_cas.exceptions import InvalidTicket
+from mama_cas.exceptions import ValidationError
 
 
 logger = logging.getLogger(__name__)
@@ -43,30 +40,49 @@ class LoginRequiredMixin(object):
         return super(LoginRequiredMixin, self).dispatch(request, *args, **kwargs)
 
 
+class CasResponseMixin(object):
+    """
+    View mixin for building CAS XML responses. Expects the view to
+    implement get_context_data() and define response_class.
+    """
+    content_type = 'text/xml'
+
+    def get(self, request, *args, **kwargs):
+        context = self.get_context_data(**kwargs)
+        return self.render_to_response(context)
+
+    def render_to_response(self, context):
+        return self.response_class(context, content_type=self.content_type)
+
+
 class ValidateTicketMixin(object):
     """
     View mixin providing ticket validation methods.
     """
     def validate_service_ticket(self, request):
         """
-        Given a ``request``, validate a service ticket string. On success, a
-        triplet is returned containing the ``ServiceTicket`` and an optional
-        ``ProxyGrantingTicket``, with no error. On error, a triplet is
-        returned containing no ``ServiceTicket`` or ``ProxyGrantingTicket``,
-        but with an ``Error`` describing what went wrong.
+        Validate a service ticket string. Return a triplet containing
+        a ``ServiceTicket`` and an optional ``ProxyGrantingTicket``,
+        or a ``ValidationError`` subclass if ticket validation failed.
         """
         service = request.GET.get('service')
         ticket = request.GET.get('ticket')
-        renew = request.GET.get('renew')
+        renew = bool(request.GET.get('renew'))
         pgturl = request.GET.get('pgtUrl')
 
         logger.debug("Service validation request received for %s" % ticket)
+
+        # Check for proxy tickets passed to /serviceValidate
+        if ticket and ticket.startswith(ProxyTicket.TICKET_PREFIX):
+            e = InvalidTicket('Proxy tickets cannot be validated'
+                              ' with /serviceValidate')
+            logger.warning("%s %s" % (e.code, e))
+            return None, None, e
+
         try:
-            st = ServiceTicket.objects.validate_ticket(ticket,
-                                                       service=service,
+            st = ServiceTicket.objects.validate_ticket(ticket, service,
                                                        renew=renew)
-        except (InvalidRequestError, InvalidTicketError,
-                InvalidServiceError, InternalError) as e:
+        except ValidationError as e:
             logger.warning("%s %s" % (e.code, e))
             return None, None, e
         else:
@@ -82,12 +98,10 @@ class ValidateTicketMixin(object):
 
     def validate_proxy_ticket(self, request):
         """
-        Given a ``request``, validate a proxy ticket string. On success, a
-        4-tuple is returned containing the ``ProxyTicket``, an optional
-        ``ProxyGrantingTicket`` and a list of all services that proxied
-        authentication, with no error. On error, a 4-tuple is returned
-        containing no ``ProxyTicket``, ``ProxyGrantingTicket`` or list of
-        proxies, but with an ``Error`` describing what went wrong.
+        Validate a proxy ticket string. Return a 4-tuple containing a
+        ``ProxyTicket``, an optional ``ProxyGrantingTicket`` and a list
+        of proxies through which authentication proceeded, or a
+        ``ValidationError`` subclass if ticket validation failed.
         """
         service = request.GET.get('service')
         ticket = request.GET.get('ticket')
@@ -95,10 +109,8 @@ class ValidateTicketMixin(object):
 
         logger.debug("Proxy validation request received for %s" % ticket)
         try:
-            pt = ProxyTicket.objects.validate_ticket(ticket,
-                                                     service=service)
-        except (InvalidRequestError, InvalidTicketError,
-                InvalidServiceError, InternalError) as e:
+            pt = ProxyTicket.objects.validate_ticket(ticket, service)
+        except ValidationError as e:
             logger.warning("%s %s" % (e.code, e))
             return None, None, None, e
         else:
@@ -122,10 +134,9 @@ class ValidateTicketMixin(object):
 
     def validate_proxy_granting_ticket(self, request):
         """
-        Given a ``request``, validate a proxy granting ticket string. On
-        success, an ordered pair is returned containing a ``ProxyTicket``,
-        with no error. On error, an ordered pair is returned containing no
-        ``ProxyTicket``, but with an ``Error`` describing what went wrong.
+        Validate a proxy granting ticket string. Return an ordered
+        pair containing a ``ProxyTicket``, or a ``ValidationError``
+        subclass if ticket validation failed.
         """
         pgt = request.GET.get('pgt')
         target_service = request.GET.get('targetService')
@@ -135,8 +146,7 @@ class ValidateTicketMixin(object):
         try:
             pgt = ProxyGrantingTicket.objects.validate_ticket(pgt,
                                                               target_service)
-        except (InvalidRequestError, BadPGTError,
-                InvalidServiceError, InternalError) as e:
+        except ValidationError as e:
             logger.warning("%s %s" % (e.code, e))
             return None, e
         else:
@@ -148,26 +158,25 @@ class ValidateTicketMixin(object):
 
 class CustomAttributesMixin(object):
     """
-    View mixin for including custom user attributes in a validation response.
+    View mixin for including user attributes in a validation response.
     """
     def get_custom_attributes(self, ticket):
         """
-        Given a ``ticket``, build a list of user attributes from the ``User``
-        and/or user profile object to be returned along with a validation
-        success. The attributes are defined using two settings variables:
+        Build a list of user attributes from the ``User`` and/or user
+        profile object. The attributes are defined with two settings:
 
         ``MAMA_CAS_USER_ATTRIBUTES``
-            This is a list of name and ``User`` attribute pairs. The name can
-            be any meaningful string, while the attribute must correspond with
-            an attribute on the ``User`` object.
+            A list of name and ``User`` attribute pairs. The name can
+            be any meaningful string, while the attribute must
+            correspond with an attribute on the ``User`` object.
 
         ``MAMA_CAS_PROFILE_ATTRIBUTES``
-            This is a list of name and user profile attribute pairs. The name
-            can be any meaningful string, while the attribute must correspond
-            with an attribute on the user profile object.
+            A list of name and user profile attribute pairs. The name
+            can be any meaningful string, while the attribute must
+            correspond with an attribute on the user profile object.
 
-        One or both of the settings variables may be used, with all data
-        returned as a single list. Ordering is not guaranteed.
+        One or both of the settings variables may be used, with all
+        items returned as a single list. Ordering is not guaranteed.
         """
         if not ticket:
             return None
@@ -177,11 +186,9 @@ class CustomAttributesMixin(object):
         user_attr_list = getattr(settings, 'MAMA_CAS_USER_ATTRIBUTES', {})
         for name, attr in user_attr_list.items():
             try:
-                attribute = [name, getattr(user, attr)]
+                attributes.append((name, getattr(user, attr)))
             except AttributeError:
                 logger.error("User has no attribute named '%s'" % attr)
-            else:
-                attributes.append(attribute)
 
         try:
             profile = user.get_profile()
@@ -192,11 +199,9 @@ class CustomAttributesMixin(object):
                                         'MAMA_CAS_PROFILE_ATTRIBUTES', {})
             for name, attr in profile_attr_list.items():
                 try:
-                    attribute = [name, getattr(profile, attr)]
+                    attributes.append((name, getattr(profile, attr)))
                 except AttributeError:
                     logger.error("Profile has no attribute named '%s'" % attr)
-                else:
-                    attributes.append(attribute)
 
         return attributes
 
@@ -207,14 +212,12 @@ class LogoutUserMixin(object):
     """
     def logout_user(self, request):
         """
-        Given a ``request``, end a single sign-on session for the current
-        user. This process occurs in two steps:
+        End a single sign-on session for the current user. This process
+        occurs in two steps:
 
-        1. Consume all ``Ticket``s created for the user to ensure none of
-           them continue to be valid outside of the active session.
+        1. Consume all valid tickets created for the user.
 
-        2. Call logout() to end the active session and completely remove all
-           existing session data for the current request.
+        2. Call logout() to end the session and purge all session data.
         """
         if request.user.is_authenticated():
             ServiceTicket.objects.consume_tickets(request.user)
