@@ -9,6 +9,7 @@ import time
 
 from django.conf import settings
 from django.db import models
+from django.db.models import Q
 from django.utils import timezone
 from django.utils.crypto import get_random_string
 from django.utils.encoding import python_2_unicode_compatible
@@ -20,11 +21,11 @@ try:
 except ImportError:  # Django version < 1.5
     from django.contrib.auth.models import User
 
-from mama_cas.exceptions import InvalidRequestError
-from mama_cas.exceptions import InvalidTicketError
-from mama_cas.exceptions import InvalidServiceError
+from mama_cas.exceptions import InvalidRequest
+from mama_cas.exceptions import InvalidTicket
+from mama_cas.exceptions import InvalidService
 from mama_cas.exceptions import InternalError
-from mama_cas.exceptions import BadPGTError
+from mama_cas.exceptions import BadPgt
 from mama_cas.utils import add_query_params
 from mama_cas.utils import is_scheme_https
 from mama_cas.utils import clean_service_url
@@ -37,15 +38,15 @@ logger = logging.getLogger(__name__)
 class TicketManager(models.Manager):
     def create_ticket(self, ticket=None, **kwargs):
         """
-        Create a new ``Ticket`` with the appropriate default values. Any
-        provided arguments are passed on to the ``create()`` function.
-        Return the newly created ``Ticket``.
+        Create a new ``Ticket``. Additional arguments are passed to the
+        ``create()`` function. Return the newly created ``Ticket``.
         """
         if not ticket:
             ticket = self.create_ticket_str()
         if 'service' in kwargs:
             kwargs['service'] = clean_service_url(kwargs['service'])
-        t = self.create(ticket=ticket, created=timezone.now(), **kwargs)
+        expires = timezone.now() + timedelta(seconds=self.model.TICKET_EXPIRE)
+        t = self.create(ticket=ticket, expires=expires, **kwargs)
         logger.debug("Created %s %s" % (t.name, t.ticket))
         return t
 
@@ -59,101 +60,95 @@ class TicketManager(models.Manager):
         return "%s-%d-%s" % (prefix, int(time.time()),
                              get_random_string(length=self.model.TICKET_RAND_LEN))
 
-    def validate_ticket(self, ticket, service=None, renew=False):
+    def validate_ticket(self, ticket, service, renew=False):
         """
-        Given a ticket string, validate the corresponding ``Ticket``
-        returning the ``Ticket`` if valid. If validation fails, raise
-        an appropriate error.
-
-        If ``service`` is provided and the ticket has a service attribute,
-        the origin of the two services will be compared. Validation will only
-        succeed if the service origins match.
+        Given a ticket string and service identifier, validate the
+        corresponding ``Ticket``. If validation succeeds, return the
+        ``Ticket``. If validation fails, raise an appropriate error.
 
         If ``renew`` is provided, the validation will only succeed if the
         ticket was issued from the presentation of the user's primary
         credentials.
         """
         if not ticket:
-            raise InvalidRequestError("No ticket string provided")
+            raise InvalidRequest("No ticket string provided")
 
         if not self.model.TICKET_RE.match(ticket):
-            raise InvalidTicketError("Ticket string %s is invalid" % ticket)
+            raise InvalidTicket("Ticket string %s is invalid" % ticket)
 
         try:
             t = self.get(ticket=ticket)
         except self.model.DoesNotExist:
-            raise InvalidTicketError("Ticket %s does not exist" % ticket)
+            raise InvalidTicket("Ticket %s does not exist" % ticket)
 
         if t.is_consumed():
-            raise InvalidTicketError("%s %s has already been used" %
-                                     (t.name, ticket))
+            raise InvalidTicket("%s %s has already been used" %
+                                (t.name, ticket))
         t.consume()
 
         if t.is_expired():
-            raise InvalidTicketError("%s %s has expired" % (t.name, ticket))
+            raise InvalidTicket("%s %s has expired" % (t.name, ticket))
 
         if not service:
-            raise InvalidRequestError("No service identifier provided")
+            raise InvalidRequest("No service identifier provided")
 
         if not is_valid_service_url(service):
-            raise InvalidServiceError("Service %s is not a valid %s URL" %
-                                      (service, t.name))
+            raise InvalidService("Service %s is not a valid %s URL" %
+                                 (service, t.name))
 
         if not same_origin(t.service, service):
-            raise InvalidServiceError("%s %s for service %s is invalid for service %s" %
-                                      (t.name, ticket, t.service, service))
+            raise InvalidService("%s %s for service %s is invalid for service "
+                                 "%s" % (t.name, ticket, t.service, service))
 
         if renew and not t.is_primary():
-            raise InvalidTicketError("%s %s was not issued via primary credentials" %
-                                     (t.name, ticket))
+            raise InvalidTicket("%s %s was not issued via primary "
+                                "credentials" % (t.name, ticket))
 
         logger.debug("Validated %s %s" % (t.name, ticket))
         return t
 
     def delete_invalid_tickets(self):
         """
-        Iterate over all ``Ticket``s and delete all consumed or expired
-        ``Ticket``s. Invalid tickets are no longer valid for future
-        authentication attempts and can be safely deleted.
+        Delete consumed or expired ``Ticket``s that are not referenced
+        by other ``Ticket``s. Invalid tickets are no longer valid for
+        authentication and can be safely deleted.
 
         A custom management command is provided that executes this method
-        on all applicable models by running ``manage.py cleanupcas``. It
-        is recommended that you run this command on a regular basis to
-        prevent invalid tickets from causing storage or performance issues.
+        on all applicable models by running ``manage.py cleanupcas``.
+        This command should be run on a regular basis to prevent invalid
+        tickets from causing storage or performance issues.
         """
-        for ticket in self.all():
-            if ticket.is_consumed() or ticket.is_expired():
+        for ticket in self.filter(Q(consumed__isnull=False) |
+                                  Q(expires__lte=timezone.now())):
+            try:
                 ticket.delete()
+            except models.ProtectedError:
+                pass
 
     def consume_tickets(self, user):
         """
-        Iterate over all ``Ticket``s for a specified user and consume all
-        tickets if they are not already consumed or expired. This is used
-        when the user logs out to ensure all tickets issued for this user
-        are no longer valid for future authentication attempts.
+        Consume all valid ``Ticket``s for a specified user. This is run
+        when the user logs out to ensure all issued tickets are no longer
+        valid for future authentication attempts.
         """
-        for ticket in self.filter(user=user):
-            if not ticket.is_consumed() and not ticket.is_expired():
-                ticket.consume()
+        for ticket in self.filter(user=user, consumed__isnull=True,
+                                  expires__gt=timezone.now()):
+            ticket.consume()
 
 
 @python_2_unicode_compatible
 class Ticket(models.Model):
     """
     ``Ticket`` is an abstract base class implementing common methods
-    and fields for the assorted ticket types.
-
-    It is recommended that you do not interact directly with this model
-    or its inheritors. Instead, the provided manager contains methods
-    for creating, validating, consuming and deleting invalid ``Ticket``s.
+    and fields for CAS tickets.
     """
-    TICKET_EXPIRE = getattr(settings, 'MAMA_CAS_TICKET_EXPIRE', 5)
+    TICKET_EXPIRE = getattr(settings, 'MAMA_CAS_TICKET_EXPIRE', 90)
     TICKET_RAND_LEN = getattr(settings, 'MAMA_CAS_TICKET_RAND_LEN', 32)
     TICKET_RE = re.compile("^[A-Z]{2,3}-[0-9]{10,}-[a-zA-Z0-9]{%d}$" % TICKET_RAND_LEN)
 
     ticket = models.CharField(_('ticket'), max_length=255, unique=True)
     user = models.ForeignKey(User, verbose_name=_('user'))
-    created = models.DateTimeField(_('created'))
+    expires = models.DateTimeField(_('expires'))
     consumed = models.DateTimeField(_('consumed'), null=True)
 
     objects = TicketManager()
@@ -170,9 +165,9 @@ class Ticket(models.Model):
 
     def consume(self):
         """
-        A ``Ticket`` is consumed by populating the ``consumed`` field with
-        the current datetime. A consumed ``Ticket`` is no longer valid for
-        any future authentication attempts.
+        Consume a ``Ticket`` by populating the ``consumed`` field with
+        the current datetime. A consumed ``Ticket`` is invalid for future
+        authentication attempts.
         """
         self.consumed = timezone.now()
         self.save()
@@ -191,9 +186,7 @@ class Ticket(models.Model):
         Check a ``Ticket``s expired state. Return ``True`` if the ticket is
         expired, and ``False`` otherwise.
         """
-        if self.created + timedelta(minutes=self.TICKET_EXPIRE) <= timezone.now():
-            return True
-        return False
+        return self.expires <= timezone.now()
 
 
 class ServiceTicket(Ticket):
@@ -233,6 +226,7 @@ class ProxyTicket(Ticket):
 
     service = models.CharField(_('service'), max_length=255)
     granted_by_pgt = models.ForeignKey('ProxyGrantingTicket',
+                                       on_delete=models.PROTECT,
                                        verbose_name=_('granted by proxy-granting ticket'))
 
     class Meta:
@@ -283,7 +277,7 @@ class ProxyGrantingTicketManager(TicketManager):
         """
         # Ensure the scheme is HTTPS before proceeding
         if not is_scheme_https(pgturl):
-            raise InternalError("Proxy callback URL scheme is not HTTPS")
+            raise InternalError("Proxy callback %s is not HTTPS" % pgturl)
 
         # Connect to proxy callback URL, checking the SSL certificate
         pgturl = add_query_params(pgturl, {'pgtId': pgtid, 'pgtIou': pgtiou})
@@ -292,41 +286,44 @@ class ProxyGrantingTicketManager(TicketManager):
             r = requests.get(pgturl, verify=verify)
         except (requests.exceptions.ConnectionError,
                 requests.exceptions.SSLError) as e:
-            raise InternalError("%s" % e)
+            raise InternalError("Proxy callback %s returned %s" % (pgturl, e))
 
         # Check the returned HTTP status code
         try:
             r.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            raise InternalError("Proxy callback returned %s" % e)
+            raise InternalError("Proxy callback %s returned %s" % (pgturl, e))
 
     def validate_ticket(self, ticket, service):
         """
-        Given a ticket string, validate the corresponding ``Ticket``
-        returning the ``Ticket`` if valid. If validation fails, raise
-        an appropriate error.
+        Given a ticket string and service identifier, validate the
+        corresponding ``Ticket``. If validation succeeds, return the
+        ``Ticket``. If validation fails, raise an appropriate error.
         """
         if not ticket:
-            raise InvalidRequestError("No ticket string provided")
+            raise InvalidRequest("No ticket string provided")
 
         if not service:
-            raise InvalidRequestError("No service identifier provided")
+            raise InvalidRequest("No service identifier provided")
 
         if not self.model.TICKET_RE.match(ticket):
-            raise InvalidTicketError("Ticket string %s is invalid" % ticket)
+            raise InvalidTicket("Ticket string %s is invalid" % ticket)
 
         try:
             t = self.get(ticket=ticket)
         except self.model.DoesNotExist:
-            raise BadPGTError("Ticket %s does not exist" % ticket)
+            raise BadPgt("Ticket %s does not exist" % ticket)
 
         if t.is_consumed():
-            raise InvalidTicketError("%s %s has already been used" %
-                                     (t.name, ticket))
+            raise InvalidTicket("%s %s has already been used" %
+                                (t.name, ticket))
+
+        if t.is_expired():
+            raise InvalidTicket("%s %s has expired" % (t.name, ticket))
 
         if not is_valid_service_url(service):
-            raise InvalidServiceError("Service %s is not a valid %s URL" %
-                                      (service, t.name))
+            raise InvalidService("Service %s is not a valid %s URL" %
+                                 (service, t.name))
 
         logger.debug("Validated %s %s" % (t.name, ticket))
         return t
@@ -341,11 +338,14 @@ class ProxyGrantingTicket(Ticket):
     """
     TICKET_PREFIX = 'PGT'
     IOU_PREFIX = 'PGTIOU'
+    TICKET_EXPIRE = getattr(settings, 'SESSION_COOKIE_AGE')
 
     iou = models.CharField(_('iou'), max_length=255, unique=True)
     granted_by_st = models.ForeignKey(ServiceTicket, null=True, blank=True,
+                                      on_delete=models.PROTECT,
                                       verbose_name=_('granted by service ticket'))
     granted_by_pt = models.ForeignKey(ProxyTicket, null=True, blank=True,
+                                      on_delete=models.PROTECT,
                                       verbose_name=_('granted by proxy ticket'))
 
     objects = ProxyGrantingTicketManager()
