@@ -14,10 +14,10 @@ except ImportError:  # Django version < 1.5
 from mama_cas.models import ProxyGrantingTicket
 from mama_cas.models import ProxyTicket
 from mama_cas.models import ServiceTicket
-from mama_cas.exceptions import BadPGTError
-from mama_cas.exceptions import InvalidRequestError
-from mama_cas.exceptions import InvalidServiceError
-from mama_cas.exceptions import InvalidTicketError
+from mama_cas.exceptions import BadPgt
+from mama_cas.exceptions import InvalidRequest
+from mama_cas.exceptions import InvalidService
+from mama_cas.exceptions import InvalidTicket
 
 
 class ServiceTicketTests(TestCase):
@@ -30,11 +30,9 @@ class ServiceTicketTests(TestCase):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
+        self.user = User.objects.create_user('ellen',
                                              password='mamas&papas',
                                              email='ellen@example.com')
-        self.ticket_info = {'service': self.service_url, 'user': self.user}
-
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
         settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
@@ -51,7 +49,8 @@ class ServiceTicketTests(TestCase):
         a valid ticket string, be neither consumed or expired and be
         related to the ``User`` that authorized its creation.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
 
         self.assertEqual(ServiceTicket.objects.count(), 1)
         self.assertTrue(re.search('^ST-[0-9]{10,}-[a-zA-Z0-9]{32}$',
@@ -66,9 +65,10 @@ class ServiceTicketTests(TestCase):
         provided with a valid ticket string and data. The validation
         process also ought to consume the ``ServiceTicket``.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
         self.assertTrue(ServiceTicket.objects.validate_ticket(st.ticket,
-                                                              service=self.service_url), st)
+                                                              self.service_url), st)
         self.assertTrue(ServiceTicket.objects.get(ticket=st.ticket).is_consumed())
 
     def test_validate_ticket_no_ticket(self):
@@ -76,18 +76,19 @@ class ServiceTicketTests(TestCase):
         The ``ServiceTicket`` validation process ought to fail when
         no ticket string is provided.
         """
-        self.assertRaises(InvalidRequestError,
-                          ServiceTicket.objects.validate_ticket, False)
+        self.assertRaises(InvalidRequest,
+                          ServiceTicket.objects.validate_ticket, None, None)
 
     def test_validate_ticket_no_service(self):
         """
         The ``ServiceTicket`` validation process ought to fail and
         consume the ticket when no service identifier is provided.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidRequestError,
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
+        self.assertRaises(InvalidRequest,
                           ServiceTicket.objects.validate_ticket,
-                          st.ticket, service=None)
+                          st.ticket, None)
         self.assertTrue(ServiceTicket.objects.get(ticket=st.ticket).is_consumed())
 
     def test_validate_ticket_invalid_ticket(self):
@@ -95,19 +96,17 @@ class ServiceTicketTests(TestCase):
         The ``ServiceTicket`` validation process ought to fail when
         an invalid ticket string is provided.
         """
-        self.assertRaises(InvalidTicketError,
-                          ServiceTicket.objects.validate_ticket,
-                          '12345', service=self.service_url)
+        self.assertRaises(InvalidTicket, ServiceTicket.objects.validate_ticket,
+                          '12345', self.service_url)
 
     def test_validate_ticket_unknown_ticket(self):
         """
         The ``ServiceTicket`` validation process ought to fail when
         a ticket cannot be found in the database.
         """
-        self.assertRaises(InvalidTicketError,
-                          ServiceTicket.objects.validate_ticket,
+        self.assertRaises(InvalidTicket, ServiceTicket.objects.validate_ticket,
                           'ST-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
-                          service=self.service_url)
+                          self.service_url)
 
     def test_validate_ticket_invalid_service(self):
         """
@@ -115,43 +114,45 @@ class ServiceTicketTests(TestCase):
         a service identifier is provided that does not match the
         ticket's service identifier.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidServiceError,
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
+        self.assertRaises(InvalidService,
                           ServiceTicket.objects.validate_ticket,
-                          st.ticket, service='http://www.example.org/')
+                          st.ticket, 'http://www.example.org/')
 
     def test_validate_ticket_invalid_service_origin(self):
         """
         The ``ServiceTicket`` validation process ought to fail when
         the service origin does not match the ticket's service origin.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidServiceError,
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
+        self.assertRaises(InvalidService,
                           ServiceTicket.objects.validate_ticket,
-                          st.ticket, service='http://sub.example.com/')
+                          st.ticket, 'http://sub.example.com/')
 
     def test_validate_ticket_expired_ticket(self):
         """
         The ``ServiceTicket`` validation process ought to fail when
         an expired ticket is provided.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        st.created = now() - timedelta(minutes=st.TICKET_EXPIRE + 1)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
+        st.expires = now() - timedelta(seconds=1)
         st.save()
-        self.assertRaises(InvalidTicketError,
-                          ServiceTicket.objects.validate_ticket,
-                          st.ticket, service=self.service_url)
+        self.assertRaises(InvalidTicket, ServiceTicket.objects.validate_ticket,
+                          st.ticket, self.service_url)
 
     def test_validate_ticket_consumed_ticket(self):
         """
         The ``ServiceTicket`` validation process ought to fail when
         a consumed ticket is provided.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
         st.consume()
-        self.assertRaises(InvalidTicketError,
-                          ServiceTicket.objects.validate_ticket,
-                          st.ticket, service=self.service_url)
+        self.assertRaises(InvalidTicket, ServiceTicket.objects.validate_ticket,
+                          st.ticket, self.service_url)
 
     def test_validate_ticket_renew(self):
         """
@@ -159,10 +160,11 @@ class ServiceTicketTests(TestCase):
         validation process should succeed if the ticket was issued from
         the presentation of the user's primary credentials.
         """
-        st = ServiceTicket.objects.create_ticket(primary=True,
-                                                 **self.ticket_info)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user,
+                                                 primary=True)
         self.assertEqual(ServiceTicket.objects.validate_ticket(st.ticket,
-                                                               service=self.service_url,
+                                                               self.service_url,
                                                                renew=True), st)
 
     def test_validate_ticket_renew_secondary(self):
@@ -171,25 +173,38 @@ class ServiceTicketTests(TestCase):
         validation process should fail if the ticket was not issued
         from the presentation of the user's primary credentials.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidTicketError,
-                          ServiceTicket.objects.validate_ticket,
-                          st.ticket, service=self.service_url, renew=True)
+        st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                 user=self.user)
+        self.assertRaises(InvalidTicket, ServiceTicket.objects.validate_ticket,
+                          st.ticket, self.service_url, renew=True)
 
     def test_invalid_ticket_deletion(self):
         """
         Calling ``delete_invalid_tickets()`` should only delete
         ``ServiceTicket``s that are either expired or consumed.
+        Invalid ``ServiceTicket``s that are referenced by
+        ``ProxyGrantingTicket``s should not be deleted.
         """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        expired_st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        expired_st.created = now() - timedelta(minutes=st.TICKET_EXPIRE + 1)
+        ServiceTicket.objects.create_ticket(service=self.service_url,
+                                            user=self.user)
+        expired_st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                         user=self.user)
+        expired_st.expires = now() - timedelta(seconds=1)
         expired_st.save()
-        consumed_st = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        consumed_st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                          user=self.user)
         consumed_st.consume()
 
+        invalid_st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                         user=self.user)
+        invalid_st.consume()
+        ProxyGrantingTicket.objects.create_ticket(self.service_url,
+                                                  granted_by_st=invalid_st,
+                                                  user=self.user,
+                                                  validate=False)
+
         ServiceTicket.objects.delete_invalid_tickets()
-        self.assertEqual(ServiceTicket.objects.count(), 1)
+        self.assertEqual(ServiceTicket.objects.count(), 2)
         self.assertRaises(ServiceTicket.DoesNotExist,
                           ServiceTicket.objects.get,
                           ticket=expired_st.ticket)
@@ -202,34 +217,15 @@ class ServiceTicketTests(TestCase):
         Calling ``consume_tickets()`` should consume tickets belonging
         to the provided user.
         """
-        st1 = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        st2 = ServiceTicket.objects.create_ticket(**self.ticket_info)
+        st1 = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                  user=self.user)
+        st2 = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                  user=self.user)
 
         ServiceTicket.objects.consume_tickets(self.user)
         self.assertTrue(ServiceTicket.objects.get(ticket=st1).is_consumed())
         self.assertTrue(ServiceTicket.objects.get(ticket=st2).is_consumed())
 
-    def test_cleanupcas_management_command(self):
-        """
-        The ``cleanupcas`` management command should only delete
-        ``ServiceTicket``s that are either expired or consumed.
-        """
-        st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        expired_st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        expired_st.created = now() - timedelta(minutes=st.TICKET_EXPIRE + 1)
-        expired_st.save()
-        consumed_st = ServiceTicket.objects.create_ticket(**self.ticket_info)
-        consumed_st.consume()
-
-        management.call_command('cleanupcas')
-        self.assertEqual(ServiceTicket.objects.count(), 1)
-        self.assertRaises(ServiceTicket.DoesNotExist,
-                          ServiceTicket.objects.get,
-                          ticket=expired_st.ticket)
-        self.assertRaises(ServiceTicket.DoesNotExist,
-                          ServiceTicket.objects.get,
-                          ticket=consumed_st.ticket)
-
 
 class ProxyTicketTests(TestCase):
     """
@@ -241,16 +237,12 @@ class ProxyTicketTests(TestCase):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
+        self.user = User.objects.create_user('ellen',
                                              password='mamas&papas',
                                              email='ellen@example.com')
         self.pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
                                                              validate=False,
                                                              user=self.user)
-        self.ticket_info = {'service': self.service_url,
-                            'user': self.user,
-                            'granted_by_pgt': self.pgt}
-
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
         settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
@@ -268,10 +260,11 @@ class ProxyTicketTests(TestCase):
         be related to the ``User`` and ``ProxyGrantingTicket`` that
         authorized its creation.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
         self.assertEqual(ProxyTicket.objects.count(), 1)
-        self.assertTrue(re.search('^PT-[0-9]{10,}-[a-zA-Z0-9]{32}$',
-                                  pt.ticket))
+        self.assertTrue(re.search('^PT-[0-9]{10,}-[a-zA-Z0-9]{32}$', pt.ticket))
         self.assertFalse(pt.is_consumed())
         self.assertFalse(pt.is_expired())
         self.assertEqual(pt.user, self.user)
@@ -283,9 +276,11 @@ class ProxyTicketTests(TestCase):
         provided with a valid ticket string and data. The validation
         process also ought to consume the ``ProxyTicket``.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
         self.assertTrue(ProxyTicket.objects.validate_ticket(pt.ticket,
-                                                            service=self.service_url), pt)
+                                                            self.service_url), pt)
         self.assertTrue(ProxyTicket.objects.get(ticket=pt.ticket).is_consumed())
 
     def test_validate_ticket_no_ticket(self):
@@ -293,8 +288,8 @@ class ProxyTicketTests(TestCase):
         The ``ProxyTicket`` validation process ought to fail when
         no ticket string is provided.
         """
-        self.assertRaises(InvalidRequestError,
-                          ProxyTicket.objects.validate_ticket, False)
+        self.assertRaises(InvalidRequest, ProxyTicket.objects.validate_ticket,
+                          None, None)
 
     def test_validate_ticket_no_service(self):
         """
@@ -302,10 +297,11 @@ class ProxyTicketTests(TestCase):
         no service identifier is provided and the ticket ought to
         be consumed.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidRequestError,
-                          ProxyTicket.objects.validate_ticket,
-                          pt.ticket, service=None)
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
+        self.assertRaises(InvalidRequest, ProxyTicket.objects.validate_ticket,
+                          pt.ticket, None)
         self.assertTrue(ProxyTicket.objects.get(ticket=pt.ticket).is_consumed())
 
     def test_validate_ticket_invalid_ticket(self):
@@ -313,18 +309,17 @@ class ProxyTicketTests(TestCase):
         The ``ProxyTicket`` validation process ought to fail when
         an invalid ticket string is provided.
         """
-        self.assertRaises(InvalidTicketError, ProxyTicket.objects.validate_ticket,
-                          '12345', service=self.service_url)
+        self.assertRaises(InvalidTicket, ProxyTicket.objects.validate_ticket,
+                          '12345', self.service_url)
 
     def test_validate_ticket_unknown_ticket(self):
         """
         The ``ProxyTicket`` validation process ought to fail when
         a ticket cannot be found in the database.
         """
-        self.assertRaises(InvalidTicketError,
-                          ProxyTicket.objects.validate_ticket,
+        self.assertRaises(InvalidTicket, ProxyTicket.objects.validate_ticket,
                           'PT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
-                          service=self.service_url)
+                          self.service_url)
 
     def test_validate_ticket_invalid_service(self):
         """
@@ -332,63 +327,82 @@ class ProxyTicketTests(TestCase):
         a service identifier is provided that does not match the
         ticket's service identifier.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidServiceError,
-                          ProxyTicket.objects.validate_ticket,
-                          pt.ticket, service='http://www.example.org/')
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
+        self.assertRaises(InvalidService, ProxyTicket.objects.validate_ticket,
+                          pt.ticket, 'http://www.example.org/')
 
     def test_validate_ticket_invalid_service_origin(self):
         """
         The ``ProxyTicket`` validation process ought to fail when
         the service origin does not match the ticket's service origin.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        self.assertRaises(InvalidServiceError,
-                          ProxyTicket.objects.validate_ticket,
-                          pt.ticket, service='http://sub.example.com/')
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
+        self.assertRaises(InvalidService, ProxyTicket.objects.validate_ticket,
+                          pt.ticket, 'http://sub.example.com/')
 
     def test_validate_ticket_expired_ticket(self):
         """
         The ``ProxyTicket`` validation process ought to fail when
         an expired ticket is provided.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        pt.created = now() - timedelta(minutes=pt.TICKET_EXPIRE + 1)
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
+        pt.expires = now() - timedelta(seconds=1)
         pt.save()
-        self.assertRaises(InvalidTicketError,
-                          ProxyTicket.objects.validate_ticket,
-                          pt.ticket, service=self.service_url)
+        self.assertRaises(InvalidTicket, ProxyTicket.objects.validate_ticket,
+                          pt.ticket, self.service_url)
 
     def test_validate_ticket_consumed_ticket(self):
         """
         The ``ProxyTicket`` validation process ought to fail when
         a consumed ticket is provided.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
+        pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                               user=self.user,
+                                               granted_by_pgt=self.pgt)
         pt.consume()
-        self.assertRaises(InvalidTicketError,
-                          ProxyTicket.objects.validate_ticket,
-                          pt.ticket, service=self.service_url)
+        self.assertRaises(InvalidTicket, ProxyTicket.objects.validate_ticket,
+                          pt.ticket, self.service_url)
 
     def test_invalid_ticket_deletion(self):
         """
         Calling ``delete_invalid_tickets()`` should only delete
         ``ProxyTicket``s that are either expired or consumed.
+        Invalid ``ProxyTicket``s that are referenced by
+        ``ProxyGrantingTicket``s should not be deleted.
         """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        expired_pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        expired_pt.created = now() - timedelta(minutes=pt.TICKET_EXPIRE + 1)
+        ProxyTicket.objects.create_ticket(service=self.service_url,
+                                          user=self.user,
+                                          granted_by_pgt=self.pgt)
+        expired_pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                       user=self.user,
+                                                       granted_by_pgt=self.pgt)
+        expired_pt.expires = now() - timedelta(seconds=1)
         expired_pt.save()
-        consumed_pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
+        consumed_pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                        user=self.user,
+                                                        granted_by_pgt=self.pgt)
         consumed_pt.consume()
 
+        invalid_pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                       user=self.user,
+                                                       granted_by_pgt=self.pgt)
+        invalid_pt.consume()
+        ProxyGrantingTicket.objects.create_ticket(self.service_url,
+                                                  granted_by_pt=invalid_pt,
+                                                  user=self.user,
+                                                  validate=False)
+
         ProxyTicket.objects.delete_invalid_tickets()
-        self.assertEqual(ProxyTicket.objects.count(), 1)
-        self.assertRaises(ProxyTicket.DoesNotExist,
-                          ProxyTicket.objects.get,
+        self.assertEqual(ProxyTicket.objects.count(), 2)
+        self.assertRaises(ProxyTicket.DoesNotExist, ProxyTicket.objects.get,
                           ticket=expired_pt.ticket)
-        self.assertRaises(ProxyTicket.DoesNotExist,
-                          ProxyTicket.objects.get,
+        self.assertRaises(ProxyTicket.DoesNotExist, ProxyTicket.objects.get,
                           ticket=consumed_pt.ticket)
 
     def test_consume_tickets(self):
@@ -396,50 +410,30 @@ class ProxyTicketTests(TestCase):
         Calling ``consume_tickets()`` should consume tickets belonging
         to the provided user.
         """
-        pt1 = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        pt2 = ProxyTicket.objects.create_ticket(**self.ticket_info)
-
+        pt1 = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                user=self.user,
+                                                granted_by_pgt=self.pgt)
+        pt2 = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                user=self.user,
+                                                granted_by_pgt=self.pgt)
         ProxyTicket.objects.consume_tickets(self.user)
         self.assertTrue(ProxyTicket.objects.get(ticket=pt1).is_consumed())
         self.assertTrue(ProxyTicket.objects.get(ticket=pt2).is_consumed())
 
-    def test_cleanupcas_management_command(self):
-        """
-        The ``cleanupcas`` management command should only delete
-        ``ProxyTicket``s that are either expired or consumed.
-        """
-        pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        expired_pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        expired_pt.created = now() - timedelta(minutes=pt.TICKET_EXPIRE + 1)
-        expired_pt.save()
-        consumed_pt = ProxyTicket.objects.create_ticket(**self.ticket_info)
-        consumed_pt.consume()
-
-        management.call_command('cleanupcas')
-        self.assertEqual(ProxyTicket.objects.count(), 1)
-        self.assertRaises(ProxyTicket.DoesNotExist,
-                          ProxyTicket.objects.get,
-                          ticket=expired_pt.ticket)
-        self.assertRaises(ProxyTicket.DoesNotExist,
-                          ProxyTicket.objects.get,
-                          ticket=consumed_pt.ticket)
-
 
 class ProxyGrantingTicketTests(TestCase):
     """
     Test the model and manager used for ``ProxyGrantingTicket``s.
     """
-    service_url = 'http://www.example.com/'
+    pgt_url = 'http://www.example.com/'
 
     def setUp(self):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
+        self.user = User.objects.create_user('ellen',
                                              password='mamas&papas',
                                              email='ellen@example.com')
-        self.ticket_info = {'user': self.user}
-
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
         settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
@@ -456,9 +450,9 @@ class ProxyGrantingTicketTests(TestCase):
         with a valid ticket string, be neither consumed or expired and
         be related to the ``User`` that authorized its creation.
         """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                        validate=False,
-                                                        **self.ticket_info)
+        pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                        user=self.user,
+                                                        validate=False)
         self.assertEqual(ProxyGrantingTicket.objects.count(), 1)
         self.assertTrue(re.search('^PGT-[0-9]{10,}-[a-zA-Z0-9]{32}$',
                                   pgt.ticket))
@@ -475,11 +469,11 @@ class ProxyGrantingTicketTests(TestCase):
         validation process should not consume the
         ``ProxyGrantingTicket``.
         """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                        validate=False,
-                                                        **self.ticket_info)
+        pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                        user=self.user,
+                                                        validate=False)
         self.assertTrue(ProxyGrantingTicket.objects.validate_ticket(pgt.ticket,
-                                                                    service=self.service_url), pgt)
+                                                                    self.pgt_url), pgt)
         self.assertFalse(pgt.is_consumed())
 
     def test_validate_ticket_no_ticket(self):
@@ -487,7 +481,7 @@ class ProxyGrantingTicketTests(TestCase):
         The ``ProxyGrantingTicket`` validation process ought to fail
         when no ticket string is provided.
         """
-        self.assertRaises(InvalidRequestError,
+        self.assertRaises(InvalidRequest,
                           ProxyGrantingTicket.objects.validate_ticket,
                           False, False)
 
@@ -496,63 +490,86 @@ class ProxyGrantingTicketTests(TestCase):
         The ``ProxyGrantingTicket`` validation process ought to fail
         when no service identifier is provided.
         """
-        self.assertRaises(InvalidRequestError,
+        self.assertRaises(InvalidRequest,
                           ProxyGrantingTicket.objects.validate_ticket,
                           'PGT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
-                          service=None)
+                          None)
 
     def test_validate_ticket_invalid_ticket(self):
         """
         The ``ProxyGrantingTicket`` validation process ought to fail
         when an invalid ticket string is provided.
         """
-        self.assertRaises(InvalidTicketError,
+        self.assertRaises(InvalidTicket,
                           ProxyGrantingTicket.objects.validate_ticket,
-                          '12345', service=self.service_url)
+                          '12345', self.pgt_url)
 
     def test_validate_ticket_unknown_ticket(self):
         """
         The ``ProxyGrantingTicket`` validation process ought to fail
         when a ticket cannot be found in the database.
         """
-        self.assertRaises(BadPGTError,
-                          ProxyGrantingTicket.objects.validate_ticket,
+        self.assertRaises(BadPgt, ProxyGrantingTicket.objects.validate_ticket,
                           'PGT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
-                          service=self.service_url)
+                          self.pgt_url)
+
+    def test_validate_ticket_expired_ticket(self):
+        """
+        The ``ProxyGrantingTicket`` validation process ought to fail
+        when an expired ticket is provided.
+        """
+        pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                        user=self.user,
+                                                        validate=False)
+        pgt.expires = now() - timedelta(seconds=1)
+        pgt.save()
+        self.assertRaises(InvalidTicket,
+                          ProxyGrantingTicket.objects.validate_ticket,
+                          pgt.ticket, self.pgt_url)
 
     def test_validate_ticket_consumed_ticket(self):
         """
         The ``ProxyGrantingTicket`` validation process ought to fail
         when a consumed ticket is provided.
         """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                        validate=False,
-                                                        **self.ticket_info)
+        pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                        user=self.user,
+                                                        validate=False)
         pgt.consume()
-        self.assertRaises(InvalidTicketError,
+        self.assertRaises(InvalidTicket,
                           ProxyGrantingTicket.objects.validate_ticket,
-                          pgt.ticket, service=self.service_url)
+                          pgt.ticket, self.pgt_url)
 
     def test_invalid_ticket_deletion(self):
         """
         Calling ``delete_invalid_tickets()`` should only delete
         ``ProxyGrantingTicket``s that are either expired or consumed.
+        ``ProxyGrantingTicket``s referenced by ``ProxyTicket``s
+        should not be deleted.
         """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                        validate=False,
-                                                        **self.ticket_info)
-        expired_pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                                validate=False,
-                                                                **self.ticket_info)
-        expired_pgt.created = now() - timedelta(minutes=pgt.TICKET_EXPIRE + 1)
+        ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                  user=self.user,
+                                                  validate=False)
+        expired_pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                                user=self.user,
+                                                                validate=False)
+        expired_pgt.expires = now() - timedelta(seconds=1)
         expired_pgt.save()
-        consumed_pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                                 validate=False,
-                                                                 **self.ticket_info)
+        consumed_pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                                 user=self.user,
+                                                                 validate=False)
         consumed_pgt.consume()
 
+        invalid_pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                                user=self.user,
+                                                                validate=False)
+        invalid_pgt.consume()
+        ProxyTicket.objects.create_ticket(service=self.pgt_url,
+                                          granted_by_pgt=invalid_pgt,
+                                          user=self.user)
+
         ProxyGrantingTicket.objects.delete_invalid_tickets()
-        self.assertEqual(ProxyGrantingTicket.objects.count(), 1)
+        self.assertEqual(ProxyGrantingTicket.objects.count(), 2)
         self.assertRaises(ProxyGrantingTicket.DoesNotExist,
                           ProxyGrantingTicket.objects.get,
                           ticket=expired_pgt.ticket)
@@ -565,40 +582,51 @@ class ProxyGrantingTicketTests(TestCase):
         Calling ``consume_tickets()`` should consume tickets belonging
         to the provided user.
         """
-        pgt1 = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                         validate=False,
-                                                         **self.ticket_info)
-        pgt2 = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                         validate=False,
-                                                         **self.ticket_info)
+        pgt1 = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                         user=self.user,
+                                                         validate=False)
+        pgt2 = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                         user=self.user,
+                                                         validate=False)
 
         ProxyGrantingTicket.objects.consume_tickets(self.user)
         self.assertTrue(ProxyGrantingTicket.objects.get(ticket=pgt1).is_consumed())
         self.assertTrue(ProxyGrantingTicket.objects.get(ticket=pgt2).is_consumed())
 
+
+class ManagementCommandTests(TestCase):
+    """
+    Test management commands that operate on ``Ticket``s.
+    """
+    pgt_url = 'http://www.example.com/'
+
+    def setUp(self):
+        """
+        Initialize the environment for each test.
+        """
+        self.user = User.objects.create_user('ellen',
+                                             password='mamas&papas',
+                                             email='ellen@example.com')
+
     def test_cleanupcas_management_command(self):
         """
-        The ``cleanupcas`` management command should only delete
-        ``ProxyGrantingTicket``s that are either expired or consumed.
+        The ``cleanupcas`` management command should delete ``Ticket``s
+        that are either expired or consumed.
         """
-        pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
+        st = ServiceTicket.objects.create_ticket(service=self.pgt_url,
+                                                 user=self.user)
+        st.consume()
+        pgt = ProxyGrantingTicket.objects.create_ticket(self.pgt_url,
+                                                        granted_by_st=st,
                                                         validate=False,
-                                                        **self.ticket_info)
-        expired_pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                                validate=False,
-                                                                **self.ticket_info)
-        expired_pgt.created = now() - timedelta(minutes=pgt.TICKET_EXPIRE + 1)
-        expired_pgt.save()
-        consumed_pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                                 validate=False,
-                                                                 **self.ticket_info)
-        consumed_pgt.consume()
+                                                        user=self.user)
+        pgt.consume()
+        pt = ProxyTicket.objects.create_ticket(service=self.pgt_url,
+                                               granted_by_pgt=pgt,
+                                               user=self.user)
+        pt.consume()
 
         management.call_command('cleanupcas')
-        self.assertEqual(ProxyGrantingTicket.objects.count(), 1)
-        self.assertRaises(ProxyGrantingTicket.DoesNotExist,
-                          ProxyGrantingTicket.objects.get,
-                          ticket=expired_pgt.ticket)
-        self.assertRaises(ProxyGrantingTicket.DoesNotExist,
-                          ProxyGrantingTicket.objects.get,
-                          ticket=consumed_pgt.ticket)
+        self.assertEqual(ServiceTicket.objects.count(), 0)
+        self.assertEqual(ProxyGrantingTicket.objects.count(), 0)
+        self.assertEqual(ProxyTicket.objects.count(), 0)
