@@ -1,17 +1,16 @@
 from __future__ import unicode_literals
 
+import unittest
+
 try:
     from urllib.parse import quote
 except ImportError:  # pragma: no cover
     from urllib import quote
 
-from xml.etree.ElementTree import ElementTree
-from xml.etree.ElementTree import fromstring
-
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import reverse
 from django.test import TestCase
+
 try:
     from django.contrib.auth import get_user_model
     User = get_user_model()
@@ -43,12 +42,6 @@ class LoginViewTests(TestCase):
         """
         self.user = User.objects.create_user(**self.user_info)
 
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        self.user.delete()
-
     def test_login_view(self):
         """
         When called with no parameters, a ``GET`` request to the view
@@ -165,20 +158,13 @@ class WarnViewTests(TestCase):
         """
         self.user = User.objects.create_user(**self.user_info)
 
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        self.user.delete()
-
     def test_warn_view(self):
         """
         When called with no parameters and no logged in user, a ``GET``
         request to the view should simply redirect to the login view.
         """
         response = self.client.get(reverse('cas_warn'))
-        self.assertRedirects(response, reverse('cas_login'),
-                             status_code=302, target_status_code=200)
+        self.assertRedirects(response, reverse('cas_login'))
         self.assertTrue('Cache-Control' in response)
         self.assertEqual(response['Cache-Control'], 'max-age=0')
 
@@ -198,8 +184,7 @@ class WarnViewTests(TestCase):
             query_str = "?service=%s" % quote(self.service_url, '')
             response = self.client.get(reverse('cas_login') + query_str)
             self.assertEqual(self.client.session.get('warn'), True)
-            self.assertRedirects(response, reverse('cas_warn') + query_str,
-                                 status_code=302, target_status_code=200)
+            self.assertRedirects(response, reverse('cas_warn') + query_str)
 
     def test_warn_view_display(self):
         """
@@ -211,7 +196,7 @@ class WarnViewTests(TestCase):
                           password=self.user_info['password'])
         query_str = "?service=%s" % quote(self.service_url, '')
         response = self.client.get(reverse('cas_warn') + query_str)
-        self.assertContains(response, self.service_url, status_code=200)
+        self.assertContains(response, self.service_url)
         self.assertTemplateUsed(response, 'mama_cas/warn.html')
 
     def test_warn_view_warned(self):
@@ -223,8 +208,7 @@ class WarnViewTests(TestCase):
         self.client.login(username=self.user_info['username'],
                           password=self.user_info['password'])
         response = self.client.post(reverse('cas_warn'))
-        self.assertRedirects(response, reverse('cas_login') + '?warned=true',
-                             status_code=302, target_status_code=200)
+        self.assertRedirects(response, reverse('cas_login') + '?warned=true')
 
 
 class LogoutViewTests(TestCase):
@@ -251,7 +235,6 @@ class LogoutViewTests(TestCase):
         """
         Undo any modifications made to the test environment.
         """
-        self.user.delete()
         settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
         settings.MAMA_CAS_FOLLOW_LOGOUT_URL = self.old_follow_url
 
@@ -261,8 +244,7 @@ class LogoutViewTests(TestCase):
         request to the view should simply redirect to the login view.
         """
         response = self.client.get(reverse('cas_logout'))
-        self.assertRedirects(response, reverse('cas_login'),
-                             status_code=302, target_status_code=200)
+        self.assertRedirects(response, reverse('cas_login'))
         self.assertTrue('Cache-Control' in response)
         self.assertEqual(response['Cache-Control'], 'max-age=0')
 
@@ -282,8 +264,7 @@ class LogoutViewTests(TestCase):
         response = self.client.post(reverse('cas_login'), self.user_info)
         query_str = '?url=http://www.example.com'
         response = self.client.get(reverse('cas_logout') + query_str)
-        self.assertRedirects(response, reverse('cas_login'),
-                             status_code=302, target_status_code=200)
+        self.assertRedirects(response, reverse('cas_login'))
         self.assertFalse('_auth_user_id' in self.client.session)
 
     def test_logout_view_follow_url(self):
@@ -312,11 +293,11 @@ class ValidateViewTests(TestCase):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
+        self.user = User.objects.create_user('ellen',
                                              password='mamas&papas',
                                              email='ellen@example.com')
-        ticket_info = {'service': self.service_url, 'user': self.user}
-        self.st = ServiceTicket.objects.create_ticket(**ticket_info)
+        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                      user=self.user)
 
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
@@ -326,7 +307,6 @@ class ValidateViewTests(TestCase):
         """
         Undo any modifications made to the test environment.
         """
-        self.user.delete()
         settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
 
     def test_validate_view(self):
@@ -335,7 +315,7 @@ class ValidateViewTests(TestCase):
         should return a validation failure.
         """
         response = self.client.get(reverse('cas_validate'))
-        self.assertContains(response, "no\n\n", status_code=200)
+        self.assertContains(response, "no\n\n")
         self.assertEqual(response.get('Content-Type'), 'text/plain')
         self.assertTrue('Cache-Control' in response)
         self.assertEqual(response['Cache-Control'], 'max-age=0')
@@ -356,7 +336,7 @@ class ValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % ('http://www.example.org/',
                                                self.st.ticket)
         response = self.client.get(reverse('cas_validate') + query_str)
-        self.assertContains(response, "no\n\n", status_code=200)
+        self.assertContains(response, "no\n\n")
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
     def test_validate_view_invalid_ticket(self):
@@ -367,7 +347,7 @@ class ValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                     'ST-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
         response = self.client.get(reverse('cas_validate') + query_str)
-        self.assertContains(response, "no\n\n", status_code=200)
+        self.assertContains(response, "no\n\n")
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
     def test_validate_view_success(self):
@@ -379,12 +359,13 @@ class ValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_validate') + query_str)
-        self.assertContains(response, "yes\nellen\n", status_code=200)
+        self.assertContains(response, "yes\nellen\n")
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
-        # This should fail as the ticket was consumed in the preceeding test
+        # This should not validate as the ticket was consumed in the
+        # preceeding request
         response = self.client.get(reverse('cas_validate') + query_str)
-        self.assertContains(response, "no\n\n", status_code=200)
+        self.assertContains(response, "no\n\n")
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
 
@@ -399,21 +380,21 @@ class ServiceValidateViewTests(TestCase):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
-                                             password='mamas&papas',
-                                             email='ellen@example.com')
+        self.user = User.objects.create_user('ellen',
+                                             email='ellen@example.com',
+                                             password='mamas&papas')
         self.user.first_name = 'Ellen'
         self.user.last_name = 'Cohen'
         self.user.save()
-
-        ticket_info = {'service': self.service_url, 'user': self.user}
-        self.st = ServiceTicket.objects.create_ticket(**ticket_info)
+        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                      user=self.user)
 
         self.old_user_attributes = getattr(settings,
                                            'MAMA_CAS_USER_ATTRIBUTES', {})
         settings.MAMA_CAS_USER_ATTRIBUTES = {'givenName': 'first_name',
                                              'sn': 'last_name',
-                                             'email': 'email'}
+                                             'email': 'email',
+                                             'test': 'invalid'}
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
         settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
@@ -431,14 +412,7 @@ class ServiceValidateViewTests(TestCase):
         should return a validation failure.
         """
         response = self.client.get(reverse('cas_service_validate'))
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_REQUEST')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        self.assertTrue('Cache-Control' in response)
-        self.assertEqual(response['Cache-Control'], 'max-age=0')
+        self.assertContains(response, 'INVALID_REQUEST')
 
     def test_service_validate_view_post(self):
         """
@@ -456,12 +430,7 @@ class ServiceValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % ('http://www.example.org/',
                                                self.st.ticket)
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_SERVICE')
 
     def test_service_validate_view_invalid_ticket(self):
         """
@@ -471,12 +440,19 @@ class ServiceValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                     'ST-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_TICKET')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_TICKET')
+
+    def test_service_validate_view_proxy_ticket(self):
+        """
+        When passed a proxy ticket, the error should explain that
+        validation failed because a proxy ticket was provided.
+        """
+        query_str = "?service=%s&ticket=%s" % (self.service_url,
+                    'PT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
+        response = self.client.get(reverse('cas_service_validate') + query_str)
+        self.assertContains(response, 'INVALID_TICKET')
+        self.assertContains(response, 'Proxy tickets cannot be validated'
+                                      ' with /serviceValidate')
 
     def test_service_validate_view_success(self):
         """
@@ -488,22 +464,17 @@ class ServiceValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
+        self.assertContains(response, 'ellen')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
 
-        # This should fail as the ticket was consumed in the preceeding test
+        # This should not validate as the ticket was consumed in the
+        # preceeding request
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_TICKET')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_TICKET')
 
+    @unittest.skipIf(pgt_url == 'https://www.example.com/',
+                     'set pgt_url to a valid HTTPS URL')
     def test_service_validate_view_pgturl(self):
         """
         When called with correct parameters and a ``pgtUrl`` parameter,
@@ -513,23 +484,12 @@ class ServiceValidateViewTests(TestCase):
         NOTE: this test will fail unless ``pgt_url`` is configured with
         a valid proxy callback URL.
         """
-        if self.pgt_url == 'https://www.example.com/':
-            raise ImproperlyConfigured("Set pgt_url to a valid HTTPS URL "
-                                       "to successfully run this test")
-
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.service_url,
                                                          self.st.ticket,
                                                          self.pgt_url)
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' +
-                         XMLNS + 'proxyGrantingTicket')
-        self.assertIsNotNone(elem)
+        self.assertContains(response, 'ellen')
+        self.assertContains(response, 'proxyGrantingTicket')
 
     def test_service_validate_view_pgturl_http(self):
         """
@@ -541,15 +501,8 @@ class ServiceValidateViewTests(TestCase):
                                                          self.st.ticket,
                                                          'http://www.example.com/')
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' +
-                         XMLNS + 'proxyGrantingTicket')
-        self.assertIsNone(elem)
+        self.assertContains(response, 'ellen')
+        self.assertNotContains(response, 'proxyGrantingTicket')
 
     def test_service_validate_view_user_attributes(self):
         """
@@ -557,26 +510,13 @@ class ServiceValidateViewTests(TestCase):
         file, a service validation success should include the list of
         configured user attributes.
         """
-        attr_names = list(settings.MAMA_CAS_USER_ATTRIBUTES.keys())
+        attr_format = getattr(settings, 'MAMA_CAS_ATTRIBUTE_FORMAT', 'jasig')
+        settings.MAMA_CAS_ATTRIBUTE_FORMAT = 'jasig'
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'attributes')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        self.assertIsNotNone(elem)
-        # Because attribute order is not guaranteed, we take a list of all
-        # configured attributes, compare each attribute found to make sure
-        # it's present and then remove it from the temporary list.
-        #
-        # When done, check that the temporary list is empty to verify that
-        # all configured attributes were matched.
-        for attribute in elem:
-            attribute.tag = attribute.tag[len(XMLNS):]
-            self.assertTrue(attribute.tag in attr_names)
-            attr_names.remove(attribute.tag)
-        self.assertEqual(len(attr_names), 0)
+        self.assertContains(response, 'attributes')
+        settings.MAMA_CAS_ATTRIBUTE_FORMAT = attr_format
 
     def test_service_validate_view_invalid_service_url(self):
         """
@@ -588,12 +528,7 @@ class ServiceValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % ('http://www.example.org/',
                                                self.st.ticket)
         response = self.client.get(reverse('cas_service_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_SERVICE')
 
 
 class ProxyValidateViewTests(TestCase):
@@ -608,27 +543,28 @@ class ProxyValidateViewTests(TestCase):
         """
         Initialize the environment for each test.
         """
-        self.user = User.objects.create_user(username='ellen',
-                                             password='mamas&papas',
-                                             email='ellen@example.com')
-        self.user.first_name = 'Ellen'
+        self.user = User.objects.create_user('ellen',
+                                             email='ellen@example.com',
+                                             password='mamas&papas')
+        self.userfirst_name = 'Ellen'
         self.user.last_name = 'Cohen'
         self.user.save()
-
-        ticket_info = {'service': self.service_url, 'user': self.user}
-        self.st = ServiceTicket.objects.create_ticket(**ticket_info)
+        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                      user=self.user)
         self.pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
                                                              validate=False,
                                                              user=self.user,
                                                              granted_by_st=self.st)
-        self.pt = ProxyTicket.objects.create_ticket(granted_by_pgt=self.pgt,
-                                                    **ticket_info)
+        self.pt = ProxyTicket.objects.create_ticket(service=self.service_url,
+                                                    user=self.user,
+                                                    granted_by_pgt=self.pgt)
 
         self.old_user_attributes = getattr(settings,
                                            'MAMA_CAS_USER_ATTRIBUTES', {})
         settings.MAMA_CAS_USER_ATTRIBUTES = {'givenName': 'first_name',
                                              'sn': 'last_name',
-                                             'email': 'email'}
+                                             'email': 'email',
+                                             'test': 'invalid'}
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
         settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
@@ -637,11 +573,6 @@ class ProxyValidateViewTests(TestCase):
         """
         Undo any modifications made to the test environment.
         """
-        self.user.delete()
-        self.st.delete()
-        self.pgt.delete()
-        self.pt.delete()
-
         settings.MAMA_CAS_USER_ATTRIBUTES = self.old_user_attributes
         settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
 
@@ -651,14 +582,7 @@ class ProxyValidateViewTests(TestCase):
         should return a validation failure.
         """
         response = self.client.get(reverse('cas_proxy_validate'))
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_REQUEST')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        self.assertTrue('Cache-Control' in response)
-        self.assertEqual(response['Cache-Control'], 'max-age=0')
+        self.assertContains(response, 'INVALID_REQUEST')
 
     def test_proxy_validate_view_post(self):
         """
@@ -676,12 +600,7 @@ class ProxyValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.invalid_service,
                                                self.pt.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_SERVICE')
 
     def test_proxy_validate_view_invalid_ticket(self):
         """
@@ -691,12 +610,7 @@ class ProxyValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                     'PT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_TICKET')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_TICKET')
 
     def test_proxy_validate_view_st_success(self):
         """
@@ -708,21 +622,14 @@ class ProxyValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
+        self.assertContains(response, 'ellen')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
 
-        # This request should fail as the ticket was consumed in the preceeding test
+        # This should not validate as the ticket was consumed in the
+        # preceeding request
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_TICKET')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_TICKET')
 
     def test_proxy_validate_view_pt_success(self):
         """
@@ -734,26 +641,16 @@ class ProxyValidateViewTests(TestCase):
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.pt.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
+        self.assertContains(response, 'ellen')
+        self.assertContains(response, 'http://www.example.com')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'proxies')
-        proxy = list(elem.getiterator(XMLNS + 'proxy'))
-        self.assertEqual(len(proxy), 1)
-        self.assertEqual(proxy[0].text, 'http://www.example.com')
 
         # This second validation request attempt should fail as the
-        # ticket was consumed in the preceeding test
+        # ticket was consumed in the preceeding request
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_TICKET')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_TICKET')
+        self.assertNotContains(response, 'http://www.example.com')
 
     def test_proxy_validate_view_proxies(self):
         """
@@ -764,27 +661,20 @@ class ProxyValidateViewTests(TestCase):
         accessed.
         """
         pgt2 = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                         validate=False,
                                                          user=self.user,
-                                                         granted_by_pt=self.pt)
-        ticket_info2 = {'service': 'http://ww2.example.com/',
-                        'user': self.user}
-        pt2 = ProxyTicket.objects.create_ticket(granted_by_pgt=pgt2,
-                                                **ticket_info2)
+                                                         granted_by_pt=self.pt,
+                                                         validate=False)
+        pt2 = ProxyTicket.objects.create_ticket(service='http://ww2.example.com',
+                                                user=self.user,
+                                                granted_by_pgt=pgt2)
         query_str = "?service=%s&ticket=%s" % ('http://ww2.example.com/', pt2)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'proxies')
-        proxy = list(elem.getiterator(XMLNS + 'proxy'))
-        self.assertEqual(len(proxy), 2)
-        self.assertEqual(proxy[0].text, 'http://ww2.example.com')
-        self.assertEqual(proxy[1].text, 'http://www.example.com')
+        self.assertContains(response, 'ellen')
+        self.assertContains(response, 'http://ww2.example.com')
+        self.assertContains(response, 'http://www.example.com')
 
+    @unittest.skipIf(pgt_url == 'https://www.example.com/',
+                     'set pgt_url to a valid HTTPS URL')
     def test_proxy_validate_view_pgturl(self):
         """
         When called with correct parameters and a ``pgtUrl`` parameter,
@@ -794,23 +684,12 @@ class ProxyValidateViewTests(TestCase):
         NOTE: this test will fail unless ``pgt_url`` is configured with
         a valid proxy callback URL.
         """
-        if self.pgt_url == 'https://www.example.com/':
-            raise ImproperlyConfigured("Set pgt_url to a valid HTTPS URL "
-                                       "to successfully run this test")
-
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.service_url,
                                                          self.pt.ticket,
                                                          self.pgt_url)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' +
-                         XMLNS + 'proxyGrantingTicket')
-        self.assertIsNotNone(elem)
+        self.assertContains(response, 'ellen')
+        self.assertContains(response, 'proxyGrantingTicket')
 
     def test_proxy_validate_view_pgturl_http(self):
         """
@@ -822,15 +701,8 @@ class ProxyValidateViewTests(TestCase):
                                                          self.pt.ticket,
                                                          'http://www.example.com/')
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' + XMLNS + 'user')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.text, 'ellen')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        elem = tree.find(XMLNS + 'authenticationSuccess/' +
-                         XMLNS + 'proxyGrantingTicket')
-        self.assertIsNone(elem)
+        self.assertContains(response, 'ellen')
+        self.assertNotContains(response, 'proxyGrantingTicket')
 
     def test_proxy_validate_view_user_attributes(self):
         """
@@ -838,44 +710,25 @@ class ProxyValidateViewTests(TestCase):
         file, a proxy validation success should include the list of
         configured user attributes.
         """
-        attr_names = list(settings.MAMA_CAS_USER_ATTRIBUTES.keys())
+        attr_format = getattr(settings, 'MAMA_CAS_ATTRIBUTE_FORMAT', 'jasig')
+        settings.MAMA_CAS_ATTRIBUTE_FORMAT = 'jasig'
         query_str = "?service=%s&ticket=%s" % (self.service_url,
-                                               self.pt.ticket)
+                                               self.st.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationSuccess/' +
-                         XMLNS + 'attributes')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        self.assertIsNotNone(elem)
-        # Because attribute order is not guaranteed, we take a list of all
-        # configured attributes, compare each attribute found to make sure
-        # it's present and then remove it from the temporary list.
-        #
-        # When done, check that the temporary list is empty to verify that
-        # all configured attributes were matched.
-        for attribute in elem:
-            attribute.tag = attribute.tag[len(XMLNS):]
-            self.assertTrue(attribute.tag in attr_names)
-            attr_names.remove(attribute.tag)
-        self.assertEqual(len(attr_names), 0)
+        self.assertContains(response, 'attributes')
+        settings.MAMA_CAS_ATTRIBUTE_FORMAT = attr_format
 
     def test_proxy_validate_view_invalid_service_url(self):
         """
         When ``MAMA_CAS_VALID_SERVICES`` is defined in the settings
         file, a service string should be checked against the list of
-        valid services.  If it does not match, a proxy authentication
+        valid services. If it does not match, a proxy authentication
         failure should be returned.
         """
         query_str = "?service=%s&ticket=%s" % (self.invalid_service,
                                                self.pt.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'authenticationFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_SERVICE')
 
 
 class ProxyViewTests(TestCase):
@@ -889,15 +742,15 @@ class ProxyViewTests(TestCase):
         """
         Create a valid user and service ticket for testing purposes.
         """
-        self.user = User.objects.create_user(username='ellen',
+        self.user = User.objects.create_user('ellen',
                                              password='mamas&papas',
                                              email='ellen@example.com')
-        ticket_info = {'service': self.service_url, 'user': self.user}
-        self.st = ServiceTicket.objects.create_ticket(**ticket_info)
+        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
+                                                      user=self.user)
         self.pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                             validate=False,
                                                              user=self.user,
-                                                             granted_by_st=self.st)
+                                                             granted_by_st=self.st,
+                                                             validate=False)
 
         self.old_valid_services = getattr(settings,
                                           'MAMA_CAS_VALID_SERVICES', ())
@@ -915,14 +768,7 @@ class ProxyViewTests(TestCase):
         should return a validation failure.
         """
         response = self.client.get(reverse('cas_proxy'))
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'proxyFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_REQUEST')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
-        self.assertTrue('Cache-Control' in response)
-        self.assertEqual(response['Cache-Control'], 'max-age=0')
+        self.assertContains(response, 'INVALID_REQUEST')
 
     def test_proxy_view_post(self):
         """
@@ -939,12 +785,7 @@ class ProxyViewTests(TestCase):
         """
         query_str = "?pgt=%s" % (self.pgt.ticket)
         response = self.client.get(reverse('cas_proxy') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'proxyFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_REQUEST')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_REQUEST')
 
     def test_proxy_view_invalid_ticket(self):
         """
@@ -954,12 +795,7 @@ class ProxyViewTests(TestCase):
         query_str = "?targetService=%s&pgt=%s" % (self.service_url,
                     'PGT-0000000000-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
         response = self.client.get(reverse('cas_proxy') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'proxyFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'BAD_PGT')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'BAD_PGT')
 
     def test_proxy_view_success(self):
         """
@@ -970,10 +806,7 @@ class ProxyViewTests(TestCase):
         query_str = "?targetService=%s&pgt=%s" % (self.service_url,
                                                   self.pgt.ticket)
         response = self.client.get(reverse('cas_proxy') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'proxySuccess/' + XMLNS + 'proxyTicket')
-        self.assertIsNotNone(elem)
-        self.assertNotEqual(elem.text, 0)
+        self.assertContains(response, 'proxyTicket')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
 
@@ -987,9 +820,4 @@ class ProxyViewTests(TestCase):
         query_str = "?targetService=%s&pgt=%s" % (self.invalid_service,
                                                   self.pgt.ticket)
         response = self.client.get(reverse('cas_proxy') + query_str)
-        tree = ElementTree(fromstring(response.content))
-        elem = tree.find(XMLNS + 'proxyFailure')
-        self.assertIsNotNone(elem)
-        self.assertEqual(elem.get('code'), 'INVALID_SERVICE')
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.get('Content-Type'), 'text/xml')
+        self.assertContains(response, 'INVALID_SERVICE')
