@@ -1,3 +1,4 @@
+import logging
 import re
 
 try:
@@ -7,6 +8,11 @@ except ImportError:  # pragma: no cover
     from urlparse import parse_qsl, urlparse, urlunparse
 
 from django.conf import settings
+from django.core import urlresolvers
+from django.http import HttpResponseRedirect
+
+
+logger = logging.getLogger(__name__)
 
 
 def add_query_params(url, params):
@@ -55,3 +61,25 @@ def is_valid_service_url(url):
         if service.match(url):
             return True
     return False
+
+
+def redirect(to, *args, **kwargs):
+    """
+    Similar to the Django ``redirect`` shortcut but with altered
+    functionality. If an optional ``params`` argument is provided, the
+    dictionary items will be injected as query parameters on the
+    redirection URL.
+    """
+    params = kwargs.pop('params', {})
+
+    try:
+        to = urlresolvers.reverse(to, args=args, kwargs=kwargs)
+    except urlresolvers.NoReverseMatch:
+        if '/' not in to and '.' not in to:
+            raise
+
+    if params:
+        to = add_query_params(to, params)
+
+    logger.debug("Redirecting to %s" % to)
+    return HttpResponseRedirect(to)
