@@ -3,17 +3,15 @@ import logging
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth import login
-from django.core.urlresolvers import reverse
 from django.http import HttpResponse
-from django.shortcuts import redirect
 from django.utils.http import urlquote_plus
 from django.utils.translation import ugettext as _
 from django.views.generic import FormView
-from django.views.generic import TemplateView
 from django.views.generic import View
 
 from mama_cas.forms import LoginForm
 from mama_cas.forms import WarnForm
+from mama_cas.mixins import CasResponseMixin
 from mama_cas.mixins import CustomAttributesMixin
 from mama_cas.mixins import LoginRequiredMixin
 from mama_cas.mixins import LogoutUserMixin
@@ -21,8 +19,10 @@ from mama_cas.mixins import NeverCacheMixin
 from mama_cas.mixins import ValidateTicketMixin
 from mama_cas.models import ProxyTicket
 from mama_cas.models import ServiceTicket
-from mama_cas.utils import add_query_params
+from mama_cas.response import ValidationResponse
+from mama_cas.response import ProxyResponse
 from mama_cas.utils import is_valid_service_url
+from mama_cas.utils import redirect
 
 
 logger = logging.getLogger(__name__)
@@ -46,56 +46,44 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
         1. ``service``: the identifier of the application the client is
            accessing. We assume this identifier to be a URL.
         2. ``renew``: requires a client to present credentials
-           regardless of any existing single sign-on session. If set,
-           its value should be ``true``.
+           regardless of any existing single sign-on session.
         3. ``gateway``: causes the client to not be prompted for
-           credentials. If a single sign-on session already exists, the
-           user will be logged in. Otherwise, the user is simply
-           forwarded to the service, if specified. If set, its value
-           should be ``true``.
+           credentials. If a single sign-on session exists the user
+           will be logged in and forwarded to the specified service.
+           Otherwise, the user remains logged out and is forwarded to
+           the specified service.
         """
         service = request.GET.get('service')
-        renew = request.GET.get('renew')
-        gateway = request.GET.get('gateway')
+        renew = bool(request.GET.get('renew'))
+        gateway = bool(request.GET.get('gateway'))
         warned = request.GET.get('warned')
 
         if renew:
             logger.debug("Renew request received by credential requestor")
             self.logout_user(request)
-            login_url = add_query_params(reverse('cas_login'),
-                                         {'service': service})
-            logger.debug("Redirecting to %s" % login_url)
-            return redirect(login_url)
+            return redirect('cas_login', params={'service': service})
         elif gateway and service:
             logger.debug("Gateway request received by credential requestor")
             if request.user.is_authenticated():
                 if self.warn_user() and not warned:
-                    warn_url = add_query_params(reverse('cas_warn'),
-                                                {'service': service,
-                                                 'gateway': gateway})
-                    return redirect(warn_url)
+                    return redirect('cas_warn', params={'service': service,
+                                                        'gateway': gateway})
 
                 st = ServiceTicket.objects.create_ticket(service=service,
                                                          user=request.user)
-                service_url = add_query_params(service, {'ticket': st.ticket})
+                return redirect(service, params={'ticket': st.ticket})
             else:
-                service_url = service
-            logger.debug("Redirecting to %s" % service_url)
-            return redirect(service_url)
+                return redirect(service)
         elif request.user.is_authenticated():
             if service:
                 logger.debug("Service ticket request received "
                              "by credential requestor")
                 if self.warn_user() and not warned:
-                    warn_url = add_query_params(reverse('cas_warn'),
-                                                {'service': service})
-                    return redirect(warn_url)
+                    return redirect('cas_warn', params={'service': service})
 
                 st = ServiceTicket.objects.create_ticket(service=service,
                                                          user=request.user)
-                service_url = add_query_params(service, {'ticket': st.ticket})
-                logger.debug("Redirecting to %s" % service_url)
-                return redirect(service_url)
+                return redirect(service, params={'ticket': st.ticket})
             else:
                 msg = _("You are logged in as %s") % request.user
                 messages.success(request, msg)
@@ -151,10 +139,8 @@ class LoginView(NeverCacheMixin, LogoutUserMixin, FormView):
             st = ServiceTicket.objects.create_ticket(service=service,
                                                      user=self.request.user,
                                                      primary=True)
-            service = add_query_params(service, {'ticket': st.ticket})
-            logger.debug("Redirecting to %s" % service)
-            return redirect(service)
-        return redirect(reverse('cas_login'))
+            return redirect(service, params={'ticket': st.ticket})
+        return redirect('cas_login')
 
     def get_initial(self):
         service = self.request.GET.get('service')
@@ -173,16 +159,15 @@ class WarnView(NeverCacheMixin, LoginRequiredMixin, FormView):
 
     def form_valid(self, form):
         service = form.cleaned_data.get('service')
-        gateway = form.cleaned_data.get('gateway')
-        return redirect(add_query_params(reverse('cas_login'),
-                                         {'service': service,
-                                          'gateway': gateway,
-                                          'warned': 'true'}))
+        gateway = bool(form.cleaned_data.get('gateway'))
+        return redirect('cas_login', params={'service': service,
+                                             'gateway': gateway,
+                                             'warned': 'true'})
 
     def get_initial(self):
         initial = {}
         service = self.request.GET.get('service')
-        gateway = self.request.GET.get('gateway')
+        gateway = bool(self.request.GET.get('gateway'))
         if service:
             initial['service'] = urlquote_plus(service)
         if gateway:
@@ -216,7 +201,7 @@ class LogoutView(NeverCacheMixin, LogoutUserMixin, View):
                 return redirect(url)
             msg = _("The application provided this link to follow: %s") % url
             messages.success(request, msg)
-        return redirect(reverse('cas_login'))
+        return redirect('cas_login')
 
 
 class ValidateView(NeverCacheMixin, ValidateTicketMixin, View):
@@ -224,9 +209,10 @@ class ValidateView(NeverCacheMixin, ValidateTicketMixin, View):
     (2.4) Check the validity of a service ticket. [CAS 1.0]
 
     When both ``service`` and ``ticket`` are provided, this view
-    responds with a ``ServiceTicket`` validation success or failure.
-    Whether or not the validation succeeds, the ``ServiceTicket`` is
-    consumed, rendering it invalid for future authentication attempts.
+    responds with a plain-text response indicating a ``ServiceTicket``
+    validation success or failure. Whether or not the validation
+    succeeds, the ``ServiceTicket`` is consumed, rendering it invalid
+    for future authentication attempts.
 
     If ``renew`` is specified, validation will only succeed if the
     ``ServiceTicket`` was issued from the presentation of the user's
@@ -235,14 +221,14 @@ class ValidateView(NeverCacheMixin, ValidateTicketMixin, View):
     def get(self, request, *args, **kwargs):
         st, pgt, error = self.validate_service_ticket(request)
         if st:
-            return HttpResponse(content="yes\n%s\n" % st.user.username,
-                                content_type='text/plain')
+            content = "yes\n%s\n" % st.user.username
         else:
-            return HttpResponse(content="no\n\n", content_type='text/plain')
+            content = "no\n\n"
+        return HttpResponse(content=content, content_type='text/plain')
 
 
 class ServiceValidateView(NeverCacheMixin, ValidateTicketMixin,
-                          CustomAttributesMixin, TemplateView):
+                          CustomAttributesMixin, CasResponseMixin, View):
     """
     (2.5) Check the validity of a service ticket. [CAS 2.0]
 
@@ -260,19 +246,17 @@ class ServiceValidateView(NeverCacheMixin, ValidateTicketMixin,
     ``ProxyGrantingTicket`` if the proxy callback URL has a valid SSL
     certificate and responds with a successful HTTP status code.
     """
-    content_type = 'text/xml'
-    template_name = 'mama_cas/validate.xml'
+    response_class = ValidationResponse
 
-    def get(self, request, *args, **kwargs):
-        st, pgt, error = self.validate_service_ticket(request)
+    def get_context_data(self, **kwargs):
+        st, pgt, error = self.validate_service_ticket(self.request)
         attributes = self.get_custom_attributes(st)
-        context = {'ticket': st, 'pgt': pgt, 'error': error,
-                   'attributes': attributes}
-        return self.render_to_response(context)
+        return {'ticket': st, 'pgt': pgt, 'error': error,
+                'attributes': attributes}
 
 
 class ProxyValidateView(NeverCacheMixin, ValidateTicketMixin,
-                        CustomAttributesMixin, TemplateView):
+                        CustomAttributesMixin, CasResponseMixin, View):
     """
     (2.6) Perform the same validation tasks as ServiceValidateView and
     additionally validate proxy tickets. [CAS 2.0]
@@ -291,27 +275,24 @@ class ProxyValidateView(NeverCacheMixin, ValidateTicketMixin,
     ``ProxyGrantingTicket`` if the proxy callback URL has a valid SSL
     certificate and responds with a successful HTTP status code.
     """
-    content_type = 'text/xml'
-    template_name = 'mama_cas/validate.xml'
+    response_class = ValidationResponse
 
-    def get(self, request, *args, **kwargs):
-        ticket = request.GET.get('ticket')
+    def get_context_data(self, **kwargs):
+        ticket = self.request.GET.get('ticket')
         if not ticket or ticket.startswith(ProxyTicket.TICKET_PREFIX):
             # If no ticket parameter is present, attempt to validate it
             # anyway so the appropriate error is raised
-            t, pgt, proxies, error = self.validate_proxy_ticket(request)
+            t, pgt, proxies, error = self.validate_proxy_ticket(self.request)
             attributes = self.get_custom_attributes(t)
         else:
-            t, pgt, error = self.validate_service_ticket(request)
+            t, pgt, error = self.validate_service_ticket(self.request)
             proxies = None
             attributes = self.get_custom_attributes(t)
-
-        context = {'ticket': t, 'pgt': pgt, 'proxies': proxies,
-                   'error': error, 'attributes': attributes}
-        return self.render_to_response(context)
+        return {'ticket': t, 'pgt': pgt, 'proxies': proxies,
+                'error': error, 'attributes': attributes}
 
 
-class ProxyView(NeverCacheMixin, ValidateTicketMixin, TemplateView):
+class ProxyView(NeverCacheMixin, ValidateTicketMixin, CasResponseMixin, View):
     """
     (2.7) Provide proxy tickets to services that have acquired proxy-
     granting tickets. [CAS 2.0]
@@ -322,10 +303,8 @@ class ProxyView(NeverCacheMixin, ValidateTicketMixin, TemplateView):
     validation succeeds, a ``ProxyTicket`` will be created and included
     in the response.
     """
-    content_type = 'text/xml'
-    template_name = 'mama_cas/proxy.xml'
+    response_class = ProxyResponse
 
-    def get(self, request, *args, **kwargs):
-        pt, error = self.validate_proxy_granting_ticket(request)
-        context = {'ticket': pt, 'error': error}
-        return self.render_to_response(context)
+    def get_context_data(self, **kwargs):
+        pt, error = self.validate_proxy_granting_ticket(self.request)
+        return {'ticket': pt, 'error': error}
